@@ -2,14 +2,13 @@ import functools
 import inspect
 import json
 from typing import Callable, Union, Dict
-import warnings
 
 import backoff
 from firebase_admin import auth as firebase_auth
 from flask import request
 from google.cloud.exceptions import ServiceUnavailable
 
-from arcane.core import CLIENTS_PARAM, ALL_CLIENTS_RIGHTS, RightsLevelEnum, GOOGLE_EXCEPTIONS_TO_RETRY
+from arcane.core import CLIENTS_PARAM, ALL_CLIENTS_RIGHTS, RightsLevelEnum, GOOGLE_EXCEPTIONS_TO_RETRY, BadRequestError
 from arcane.datastore import Client as DatastoreClient
 from arcane.pubsub import Client as PubSubClient
 
@@ -95,8 +94,15 @@ def check_access_rights(service: str,
                     # Failing to send tracking info should not impact feature usability
                     pass
 
-                rights = claims.get('features_rights', {}).get(service_user_right, RightsLevelEnum.NONE)
-                if rights < required_rights:
+
+                try:
+                    _check_feature_right_access(
+                        claims,
+                        required_rights,
+                        service_user_right
+                    )
+                except BadRequestError:
+                    rights = _get_level_access_for_a_feature_from_claims(claims, service_user_right)
                     return {'detail': f"You don't have access to this resource : your rights are "
                             f"'{rights_mapper.get(str(rights), '0')}' for service '{service}'. "
                             f"You need '{rights_mapper.get(str(required_rights), '0')}'.",
@@ -143,6 +149,51 @@ def verify_token(token: str) -> Dict:
     return claims
 
 
+def check_feature_right_of_authenticated_user(
+    required_rights: int,
+    service_user_right: str,
+    auth_enabled: bool=True
+):
+    # Check feature rights
+    if not auth_enabled:
+        return
+    claims = {}
+    try:
+        token = str(request.headers.get('Authorization', '')).split(' ').pop()
+        claims = verify_token(token)
+    except (firebase_auth.ExpiredIdTokenError,
+            firebase_auth.RevokedIdTokenError,
+            firebase_auth.InvalidIdTokenError) as e:
+        if required_rights != RightsLevelEnum.NONE:
+            raise BadRequestError('You don\'t have access to Adscale resources\n')
+    except ValueError as e:
+        raise BadRequestError(str(e))
+
+    _check_feature_right_access(
+        claims,
+        required_rights,
+        service_user_right
+    )
+
+
+
+def _check_feature_right_access(
+    claims: Dict,
+    required_rights: int,
+    service_user_right: str,
+):
+    rights = _get_level_access_for_a_feature_from_claims(claims, service_user_right)
+    if rights < required_rights:
+        raise BadRequestError("You don't have access to this resource")
+
+
+def _get_level_access_for_a_feature_from_claims(
+    claims: Dict,
+    service_user_right: str
+):
+    return claims.get('features_rights', {}).get(service_user_right, RightsLevelEnum.NONE)
+
+
 def get_user_email(token: str=None, auth_enabled=True) -> Union[str, None]:
     """
     retrieves user id from connexion header.
